<?php
/**
* 2007-2020 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    patworx multimedia GmbH <service@patworx.de>
*  @copyright 2007-2020 patworx multimedia GmbH
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

define('CURRENT_MODULE_DIR_EASYCREDIT', realpath(dirname(__FILE__)));
require_once(CURRENT_MODULE_DIR_EASYCREDIT . '/vendor/autoload.php');
require_once(dirname(__FILE__) . '/classes/EasycreditLog.php');
require_once(dirname(__FILE__) . '/classes/EasycreditCalculator.php');

class Easycreditratenkauf extends PaymentModule
{
    protected $config_form = false;

    private $_postSuccess = array();
    private $_postError = array();

    protected $min_amount = 200;
    protected $max_amount = 10000;

    public static $einstieg_url = 'https://ratenkauf.easycredit.de/ratenkauf/content/intern/einstieg.jsf?vorgangskennung=';
    public static $zustimmung_url = 'https://ratenkauf.easycredit.de/ratenkauf-ws/rest/v0.4/texte/zustimmung/';
    public static $calc_url = 'https://ratenkauf.easycredit.de/ratenkauf-ws/rest/v2/modellrechnung/durchfuehren?webshopId=%s&finanzierungsbetrag=';

    public function __construct()
    {
        $this->name = 'easycreditratenkauf';
        $this->tab = 'payments_gateways';
        $this->version = '1.6.1';
        $this->author = 'TeamBank AG, developed by patworx multimedia GmbH';
        $this->need_instance = 1;
        $this->bootstrap = true;
        $this->module_key = '4100b32025aedaf0db43d3c491941f31';

        parent::__construct();

        $this->displayName = $this->l('Ratenkauf by easyCredit');
        $this->description = $this->l('Offer installment by easyCredit for your customers');

        $this->confirmUninstall = $this->l('Are you sure to uninstall this module?');

        $this->limited_countries = array('DE');

        $this->limited_currencies = array('EUR');

        $this->ps_versions_compliancy = array('min' => '1.5', 'max' => _PS_VERSION_);
    }

    public function install()
    {
        if (extension_loaded('curl') == false) {
            $this->_errors[] = $this->l('You have to enable the cURL extension on your server to install this module');
            return false;
        }

        $iso_code = Country::getIsoById(Configuration::get('PS_COUNTRY_DEFAULT'));

        if (in_array($iso_code, $this->limited_countries) == false) {
            $this->_errors[] = $this->l('This module is not available in your country');
            return false;
        }

        include(dirname(__FILE__).'/sql/install.php');

        if (version_compare(_PS_VERSION_, '1.7.0.0', '<')) {
            $this->registerHook('displayPaymentEU');
            $this->registerHook('payment');
        } else {
            Tools::clearSmartyCache();
            Tools::clearXMLCache();
            Media::clearCache();
            Tools::generateIndex();

            $sf2Refresh = new \PrestaShopBundle\Service\Cache\Refresh();
            $sf2Refresh->addCacheClear(_PS_MODE_DEV_ ? 'dev' : 'prod');
            $sf2Refresh->execute();
        }

        return parent::install() &&
            $this->registerHook('header') &&
            $this->registerHook('backOfficeHeader') &&
            $this->registerHook('paymentReturn') &&
            $this->registerHook('displayShoppingCart') &&
            $this->registerHook('paymentOptions') &&
            $this->registerHook('overrideTOSDisplay') &&
            $this->registerHook('displayAdminOrder') &&
            $this->registerHook('displayProductButtons') &&
            $this->install16Hook();
    }

    protected function install16Hook()
    {
        if (version_compare(_PS_VERSION_, '1.7.0', '<') || Tools::substr(_PS_VERSION_, 0, 3) == '1.6') {
            return $this->registerHook('payment') &&
                $this->registerHook('displayPaymentEU');
        }
        return true;
    }

    public function uninstall()
    {
        include(dirname(__FILE__).'/sql/uninstall.php');
        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        if (((bool)Tools::isSubmit('submitEasycreditratenkaufModule')) == true) {
            $this->postProcess();
            if (count($this->_postSuccess)) {
                $this->context->smarty->assign(array('postSuccess' => $this->_postSuccess));
            }
            if (count($this->_postError)) {
                $this->context->smarty->assign(array('postError' => $this->_postError));
            }
        }

        $this->context->smarty->assign('module_dir', $this->_path);
        $this->context->smarty->assign('isPS15', version_compare(_PS_VERSION_, '1.6.0', '<'));
        $this->context->smarty->assign('ec_form', $this->renderForm());
        return $this->context->smarty->fetch($this->local_path.'views/templates/admin/config_plugin.tpl');
    }

    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitEasycreditratenkaufModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    protected function getConfigForm()
    {
        return array(
            'form' => array(
                'legend' => array(
                'title' => $this->l('Settings'),
                'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-envelope"></i>',
                        'desc' => $this->l('Please enter your ratenkauf by easyCredit Webshop-ID'),
                        'name' => 'EASYCREDITRATENKAUF_API_KEY',
                        'label' => $this->l('Webshop-ID'),
                    ),
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-envelope"></i>',
                        'desc' => $this->l('Please enter your ratenkauf by easyCredit API-Password'),
                        'name' => 'EASYCREDITRATENKAUF_API_TOKEN',
                        'label' => $this->l('API-Password'),
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    protected function getConfigFormValues()
    {
        return array(
            'EASYCREDITRATENKAUF_API_KEY' => Configuration::get('EASYCREDITRATENKAUF_API_KEY', null),
            'EASYCREDITRATENKAUF_API_TOKEN' => Configuration::get('EASYCREDITRATENKAUF_API_TOKEN', null),
        );
    }

    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, trim(Tools::getValue($key)));
        }
        $this->_postSuccess[] = $this->l('Settings updated');
    }

    public function hookBackOfficeHeader()
    {
        if ((version_compare(_PS_VERSION_, '1.7.0', '<') && Tools::getValue('module_name') == $this->name)
            || (version_compare(_PS_VERSION_, '1.7.0', '>=') && Tools::getValue('configure') == $this->name)) {
            $this->context->controller->addJS($this->_path.'views/js/back.js');
            $this->context->controller->addCSS($this->_path.'views/css/back.css');
        }
        $this->context->controller->addJS($this->_path.'views/js/minimize.js');
    }

    public function hookHeader()
    {
        $htmlreturn = false;
        if (isset($this->context->cookie->easycredit_error_message)) {
            if ((bool)Configuration::get('PS_ADVANCED_PAYMENT_API') && version_compare(_PS_VERSION_, '1.7.0', '<') && (bool)Configuration::get('PS_ORDER_PROCESS_TYPE')) {
                $this->context->smarty->assign('easycredit_error', $this->context->cookie->easycredit_error_message);
                $htmlreturn = $this->display(__FILE__, 'views/templates/hook/_header_error.tpl');
            } else {
                $this->context->controller->errors[] = $this->context->cookie->easycredit_error_message;
            }
            unset($this->context->cookie->easycredit_error_message);
        }
        $this->context->controller->addJS($this->_path.'/views/js/front.js');
        $this->context->controller->addCSS($this->_path.'/views/css/front.css');
        if (version_compare(_PS_VERSION_, '1.7.0', '>=')) {
            return $this->display(__FILE__, 'views/templates/hook/header.tpl');
        } else {
            $this->context->controller->addJS('https://ratenkauf.easycredit.de/ratenkauf/js/paymentPagePlugin.js');
            if ($htmlreturn) {
                return $htmlreturn;
            }
        }
    }

    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return array();
        }
        $total = $params['cart']->getOrderTotal(true);

        if (!$this->ratenkaufIsAvailable($total)) {
            return array();
        }

        $this->smarty->assign('zustimmungtext', $this->getZustimmungText());

        $newOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $newOption->setCallToActionText($this->l('Ratenkauf by easyCredit'))
            ->setModuleName('easycreditratenkauf')
            ->setAction($this->context->link->getModuleLink($this->name, 'redirect', array(), true))
            ->setLogo(Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/views/img/rk_logo.jpg'))
            ->setAdditionalInformation($this->display(__FILE__, 'views/templates/hook/hookPaymentOptionAdditionalInfo.tpl'));
        $payment_options = array($newOption);
        return $payment_options;
    }

    public function hookPayment($params)
    {
        if (version_compare(_PS_VERSION_, '1.7.0', '>=')) {
            return false;
        }
        $currency_id = $params['cart']->id_currency;
        $currency = new Currency((int)$currency_id);

        if (in_array($currency->iso_code, $this->limited_currencies) == false) {
            return false;
        }
        $total = $params['cart']->getOrderTotal(true);
        if (!$this->ratenkaufIsAvailable($total)) {
            return false;
        }

        $this->smarty->assign('zustimmungtext', $this->getZustimmungText());

        $this->smarty->assign('module_dir', $this->_path);

        return $this->display(__FILE__, 'views/templates/hook/payment.tpl');
    }

    protected function getZustimmungText()
    {
        $json_raw = Tools::file_get_contents(self::$zustimmung_url . Configuration::get('EASYCREDITRATENKAUF_API_KEY'));
        $json = json_decode($json_raw, true);
        return $json['zustimmungDatenuebertragungPaymentPage'];
    }

    public function hookDisplayPaymentEU($params)
    {
        if (version_compare(_PS_VERSION_, '1.7.0', '>=')) {
            return false;
        }
        if (!$this->active) {
            return;
        }
        $currency_id = $params['cart']->id_currency;
        $currency = new Currency((int)$currency_id);

        if (in_array($currency->iso_code, $this->limited_currencies) == false) {
            return false;
        }
        $total = $params['cart']->getOrderTotal(true);
        if (!$this->ratenkaufIsAvailable($total)) {
            return false;
        }

        $payment_options = array(
            'cta_text' => $this->l('Ratenkauf by easyCredit') . '<span class="easycredit_hint"></span>',
            'logo' => Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/logo.png'),
            'action' => $this->context->link->getModuleLink($this->name, 'redirect', array(), true)
        );

        return $payment_options;
    }

    public function hookOverrideTOSDisplay($params)
    {
        if (!(bool)Configuration::get('PS_ADVANCED_PAYMENT_API')) {
            return false;
        }

        return $this->display(__FILE__, 'hookOverrideTOSDisplay.tpl');
    }

    public function hookPaymentReturn($params)
    {
        if ($this->active == false) {
            return;
        }

        if (version_compare(_PS_VERSION_, '1.7.0', '>=')) {
            return '';
        } else {
            $order = $params['objOrder'];

            if ($order->getCurrentOrderState()->id != Configuration::get('PS_OS_ERROR')) {
                $this->smarty->assign('status', 'ok');
            }

            $this->smarty->assign(array(
                'id_order' => $order->id,
                'reference' => $order->reference,
                'params' => $params,
                'total' => Tools::displayPrice($params['total_to_pay'], $params['currencyObj'], false),
            ));
            return $this->display(__FILE__, 'views/templates/hook/confirmation.tpl');
        }
    }

    public function hookDisplayProductButtons($params)
    {
        if (is_object($params['product']) && $params['product'] instanceof Product) {
            $this->context->smarty->assign('interest_calculator_price', (float)$params['product']->getPrice());
        } else {
            $this->context->smarty->assign('interest_calculator_price', (float)$params['product']['price_amount']);
        }

        if (!$this->ratenkaufIsAvailable($this->context->smarty->getTemplateVars('interest_calculator_price'))) {
            return false;
        }

        $this->context->smarty->assign('webshopID', Configuration::get('EASYCREDITRATENKAUF_API_KEY'));
        return $this->display(__FILE__, 'views/templates/hook/display_product_button.tpl');
    }

    public function hookDisplayShoppingCart($params)
    {
        $this->context->smarty->assign('interest_calculator_price', (float)$params['cart']->getOrderTotal(true));
        $this->context->smarty->assign('webshopID', Configuration::get('EASYCREDITRATENKAUF_API_KEY'));
        return $this->display(__FILE__, 'views/templates/hook/display_shopping_cart.tpl');
    }

    public function ratenkaufIsAvailable($amount)
    {
        try {
            $calculator = new EasycreditCalculator($amount);
            return $calculator->ratenkaufIsAvailable();
        } catch (Exception $e) {
            if ($amount > $this->max_amount ||
                $amount < $this->min_amount) {
                return false;
            }
        }
    }

    /**
     * Admin actions and log-view
     *
     * @param $params
     */
    public function hookDisplayAdminOrder($params)
    {
        $order = new Order((int) $params['id_order']);
        if ($order->module != $this->name) {
            return;
        }
        if (version_compare(_PS_VERSION_, '1.7.7', '>=')) {
            $this->context->smarty->assign('isHigher176', true);
        } else {
            $this->context->smarty->assign('isHigher176', false);
        }
        $this->context->smarty->assign('transactions', EasycreditLog::getDataForOrderId((int)$order->id));

        return $this->display(__FILE__, 'views/templates/admin/hook_order_history.tpl');
    }

    public static function createLog($ps_cart_id, $ps_customer_id, $url, $easycredit_vorgang, $request, $response)
    {
        $easycreditLog = new EasycreditLog();
        $easycreditLog->ps_cart_id = (int)$ps_cart_id;
        $easycreditLog->ps_customer_id = (int)$ps_customer_id;
        $easycreditLog->url = $url;
        $easycreditLog->easycredit_vorgang = pSQL($easycredit_vorgang);
        $easycreditLog->request = pSQL($request);
        $easycreditLog->response = pSQL($response);
        $easycreditLog->save();
    }
}
